C     -*- fortran -*-
C     This file is autogenerated with f2py (version:2.4.3)
C     It contains Fortran 77 wrappers to fortran functions.

      subroutine f2pywrapccf (ccff2pywrap, m_l, m_h, wav, spec, we
     &ight, sn, v_r, snw, n, m)
      external ccf
      double precision v_r
      double precision snw
      integer n
      integer m
      double precision m_l(n)
      double precision m_h(n)
      double precision wav(m)
      double precision spec(m)
      double precision weight(n)
      double precision sn(m)
      double precision ccff2pywrap
      double precision  ccf
      ccff2pywrap = ccf(m_l, m_h, wav, spec, weight, sn, v_r, snw,
     & n, m)
      end


      subroutine f2pywrapccfpix (ccfpixf2pywrap, m_l, m_h, x, thar
     &, delta, n, m)
      external ccfpix
      double precision delta
      integer n
      integer m
      double precision m_l(n)
      double precision m_h(n)
      double precision x(m)
      double precision thar(m)
      double precision ccfpixf2pywrap
      double precision  ccfpix
      ccfpixf2pywrap = ccfpix(m_l, m_h, x, thar, delta, n, m)
      end


      subroutine f2pywrapccfcos (ccfcosf2pywrap, m_l, m_h, wav, sp
     &ec, weight, sn, v_r, snw, n, m)
      external ccfcos
      double precision v_r
      double precision snw
      integer n
      integer m
      double precision m_l(n)
      double precision m_h(n)
      double precision wav(m)
      double precision spec(m)
      double precision weight(n)
      double precision sn(m)
      double precision ccfcosf2pywrap
      double precision  ccfcos
      ccfcosf2pywrap = ccfcos(m_l, m_h, wav, spec, weight, sn, v_r
     &, snw, n, m)
      end

